@@ -34,6 +34,9 @@ static const std::string LibTMCG_ID =
 bool init_libTMCG
 	()
 {
+	// initialize memory guards
+	CallasDonnerhackeFinneyShawThayerRFC4880::MemoryGuardReset();
+
 	// initialize libgmp
 	if (strcmp(gmp_version, TMCG_LIBGMP_VERSION) < 0)
 	{
@@ -119,14 +122,6 @@ bool init_libTMCG
 			"] not available" << std::endl;
 		return false;
 	}
-	if (gcry_md_test_algo(GCRY_MD_SHA3_256)) // check for SHA3-256
-	{
-		std::cerr << "init_libTMCG(): libgcrypt algorithm " <<
-			GCRY_MD_SHA3_256 << " [" <<
-			"SHA3-256" <<
-			"] not available" << std::endl;
-		return false;
-	}
 	if (gcry_cipher_test_algo(GCRY_CIPHER_AES)) // check for AES128
 	{
 		std::cerr << "init_libTMCG(): libgcrypt algorithm " <<
@@ -151,6 +146,17 @@ bool init_libTMCG
 			"] not available" << std::endl;
 		return false;
 	}
+#ifdef BOTAN
+	// initialize libbotan-2
+	std::string botan_rvc = Botan::runtime_version_check(BOTAN_VERSION_MAJOR,
+		BOTAN_VERSION_MINOR, BOTAN_VERSION_PATCH);
+	if (botan_rvc != "")
+	{
+		std::cerr << "init_libTMCG(): libbotan-2 runtime version check" <<
+			" failed with \"" << botan_rvc << "\"" << std::endl;
+		return false;
+	}
+#endif
 	return true;
 }
 
