@@ -1,7 +1,7 @@
 /*******************************************************************************
    This file is part of LibTMCG.
 
- Copyright (C) 2016, 2017  Heiko Stamer <HeikoStamer@gmx.net>
+ Copyright (C) 2016, 2017, 2018  Heiko Stamer <HeikoStamer@gmx.net>
 
    LibTMCG is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -119,7 +119,7 @@ void start_instance
 			tcom = new PedersenTrapdoorCommitmentScheme(vtmf->p, vtmf->q, vtmf->k, vtmf->g);
 			assert(tcom->CheckGroup());
 			mpz_init(c), mpz_init(r), mpz_init(m);
-			mpz_wrandomm(m, vtmf->p);
+			tmcg_mpz_wrandomm(m, vtmf->p);
 			tcom->Commit(c, r, m);
 			assert(tcom->Verify(c, r, m));
 			delete tcom;
@@ -307,8 +307,8 @@ int main
 {
 	assert(init_libTMCG());
 
-	BarnettSmartVTMF_dlog 	*vtmf;
-	std::stringstream 	crs;
+	BarnettSmartVTMF_dlog *vtmf;
+	std::stringstream crs;
 
 	// create and check VTMF instance
 	std::cout << "BarnettSmartVTMF_dlog()" << std::endl;
@@ -322,6 +322,9 @@ int main
 	// publish VTMF instance as string stream (common reference string)
 	std::cout << "vtmf.PublishGroup(crs)" << std::endl;
 	vtmf->PublishGroup(crs);
+
+	// release VTMF instance
+	delete vtmf;
 	
 	// test case #1: all correct
 	init();
@@ -329,7 +332,7 @@ int main
 		start_instance(crs, i, false);
 	if (!done())
 		return 1;
-	
+/*	
 	// test case #2: two corrupted parties
 	init();
 	for (size_t i = 0; i < N; i++)
@@ -341,9 +344,7 @@ int main
 	}
 	if (!done())
 		return 1;
-	
-	// release VTMF instance
-	delete vtmf;
+*/
 	
 	return 0;
 }
